/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Jim McDaniel                                                */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include <string.h>

#include "rsffile.h"
#include "htmltmpl.h"
#include "rsf2html.h"

/* Parameters passed in by the command line switchs */
enum ArgType {
  ArgNone, ArgError, ArgHelp, ArgMultiarc = 4, ArgLoc = 8, Arg4 = 16,
  ArgA = 32, ArgB = 64, ArgS = 128, ArgP = 256, ArgT = 512, ArgX = 1024
};

static int ParseCmdLine(int argc, char **argv) {
  int i, j, k, args = 0;
  int isLast;
  char c;

  for (i = 1; i < argc; i++) {
    if (*argv[i] != '-') {
      fprintf(stderr, "Invalid argument \"%s\"\n", argv[i]);
      return (ArgError | ArgHelp);
    }
    k = strlen(argv[i]);
    for (j = 1; j < k; j++) {
      c = *(argv[i] + j);
      isLast = !(*(argv[i] + j + 1));	/* last character in argument? */
      switch (c) {
      case 'h':
	args |= (int)ArgHelp;
	break;
      case 'm':
	args |= (int)ArgMultiarc;
	break;
      case 'l':
	args |= (int)ArgLoc;
	break;
      case '4':
	args |= (int)Arg4;
	break;

      case 'a':
	if (isLast && i + 1 < argc && j + 1 == k && argv[i + 1][0] != '-') {
	  args |= (int)ArgA;
	  char *c = strdup(argv[++i]);
	  char *d = c;
	  char *e;
	  while ((e = strchr(d, ',')) != NULL) {
	    *e = '\0';
	    HtmlRsfTuple::addAnchorString(UniqueString(d));
	    d = e + 1;
	  }
	  HtmlRsfTuple::addAnchorString(UniqueString(d));
	  free(c);
	  break;
	}
	fprintf(stderr, "One or more anchor types must be specified\n");
	return (ArgError | ArgHelp);

      case 'b':
	if (isLast && i + 1 < argc && j + 1 == k && argv[i + 1][0] != '-') {
	  args |= (int)ArgB;
	  char *c = strdup(argv[++i]);
	  char *d = c;
	  char *e;
	  while ((e = strchr(d, ',')) != NULL) {
	    *e = '\0';
	    HtmlRsfTuple::addBookmarkString(UniqueString(d));
	    d = e + 1;
	  }
	  HtmlRsfTuple::addBookmarkString(UniqueString(d));
	  free(c);
	  break;
	}
	fprintf(stderr, "One or more bookmark types must be specified\n");
	return (ArgError | ArgHelp);

      case 's':
	if (isLast && i + 1 < argc && j + 1 == k && argv[i + 1][0] != '-') {
	  maxLines = atoi(argv[++i]);
	  if (maxLines < 1) {
	    fprintf(stderr, "-l must be followed by a non-negative number of lines\n");
	    return (ArgError | ArgHelp);
	  }
	}
	args |= (int)ArgS;
	break;

      case 'p':
	args |= (int)ArgP;
	break;

      case 't':
	if (isLast && i + 1 < argc && j + 1 == k && argv[i + 1][0] != '-') {
	  args |= (int)ArgT;
	  if (!HtLoadTemplate(argv[++i])) return (ArgError | ArgHelp);
	  break;
	}
	fprintf(stderr, "-t must be followed by a template file name\n");
	return (ArgError | ArgHelp);

      case 'x':
	if (isLast && i + 1 < argc && j + 1 == k && argv[i + 1][0] != '-') {
	  tabSize = atoi(argv[++i]);
	  if (tabSize > 10 || tabSize < 1) {
	    fprintf(stderr, "-x expands tabs to a maximum of 10 spaces\n");
	    return (ArgError | ArgHelp);
	  }
	} else {
	  tabSize = 4;
	}
	args |= (int)ArgX;
	break;

      default:
	fprintf(stderr, "Invalid argument \"%s\"\n", argv[i]);
	return (ArgError | ArgHelp);
      }
    }
  }
  return args;
}

static void PrintHelp(char *progName) {
  fprintf(stderr,
	  "HTLMize Source Using Rigi Standard Format (htmlrsf) - " VERSION "\n");
  fprintf(stderr,
	  "Usage: %s\n -a anchor[,anchor,...] -b bookmark[,bookmark,...]\n"
	  " [-hlm4] [-t template] [-s maxLines] [-x numSpaces]\n", progName);
  fprintf(stderr,
	  "\t-l\t\t combine multiple \"loc\" attributes that reference the same object\n");
  fprintf(stderr,
	  "\t-m\t\t combine duplicate arcs into single \"multiarc\" type\n");
  fprintf(stderr,
	  "\t-4\t\t preserve 4-tuples (overrides -l and -m)\n");
  fprintf(stderr,
	  "\t-a anchor\tlist of RSF identifiers that are anchors\n");
  fprintf(stderr,
	  "\t-b bookmark\tlist of RSF identifiers that are bookmarks\n");
  fprintf(stderr,
	  "\t-h\t\thelp\n");
  fprintf(stderr,
	  "\t-s maxLines\tsegment source files at a maximum size of maxLines\n");
  fprintf(stderr,
	  "\t-p\t\tapply \"preformatted\" (<pre>) HTML tags\n");
  fprintf(stderr,
	  "\t-t template\tuse HTML template file\n");
  fprintf(stderr,
	  "\t-x numSpaces\texpand tabs to spaces (default is 4 spaces)\n");
}

int main(int argc, char *argv[]) {
  int args = ParseCmdLine(argc, argv);
  if (!(args & ArgHelp)) {
    if (!(args & ArgA)) {
      fprintf(stderr, "Command line argument \"-a\" must be specified\n");
      args |= (ArgError | ArgHelp);
    }
    if (!(args & ArgB)) {
      fprintf(stderr, "Command line argument \"-b\" must be specified\n");
      args |= (ArgError | ArgHelp);
    }
  }
  if (args & ArgHelp) {
    PrintHelp(argv[0]);
    return (args & ArgError);
  }

  /* If 4-tuples are preserved, override -l and -m switches */
  if (args & Arg4) {
    args &= ~ArgLoc;
    args &= ~ArgMultiarc;
  }

  htmlTupleSet tuples;

  if (!readInput(tuples, true))
    return 1;

  if (!Htmlize(tuples, args & ArgS, args & ArgP))
    return 1;

  /* Sort the set of tuples into sortrsf order */
  rsfTupleSet newtuples;
  for (htmlTupleSet::iterator iter = tuples.begin();
       iter != tuples.end(); ++iter)
    newtuples.insert(*iter);

  return !writeOutput(newtuples, args & Arg4, args & ArgLoc, args & ArgMultiarc);
}
