/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Johannes Martin (jmartin@csr.csc.uvic.ca)                   */
/*                                                                     */
/***********************************************************************/
#ifndef __RSFFILE_H__
#define __RSFFILE_H__

#include "rsftuple.h"

/*
 * Read the contents of an RSF file from input into a set of tuples.
 * Remove the 4th element in the line, if preserve4Tuple is not set.
 * Errors and RSF comments will be printed to stderr, unless forgiving 
 * is true.
 * Return false if fgets failes, otherwise true.
 */
template <class Tuples> int readInput(FILE *input,
				      Tuples& tuples, 
				      bool preserve4Tuple,
				      bool forgiving = false) {
  int recLen = 1024;
  char *rec = new char[recLen];
  unsigned long lineNo = 0;

  while (fgets(rec, recLen, input) != NULL) {
    while (rec[strlen(rec)-1] != '\n') {
      char *tmp = new char[recLen *= 2];
      strcpy(tmp, rec);
      delete[] rec;
      rec = tmp;
      if (fgets(rec + strlen(rec), recLen - strlen(rec), input) == NULL)
        break;
    }

    RsfTuple rsfTuple(rec, ++lineNo, preserve4Tuple, forgiving);
    if (rsfTuple.isValid())
      tuples.insert(rsfTuple);
  }
  delete[] rec;
  return feof(input);
}

template <class Tuples> int readInput(Tuples& tuples, 
				      bool preserve4Tuple,
				      bool forgiving = false) {
  readInput(stdin, tuples, preserve4Tuple, forgiving);
}

/*
 * Write a set of tuples to output.
 * Remove the 4th element in the line, if preserve4Tuple is not Tuples.
 * Return false if printf failes, otherwise true.
 */
template <class Tuples> int writeOutput(FILE *output,
					Tuples& tuples, 
					bool preserve4Tuple,
					bool argLoc, 
					bool argMulti) {
  typename Tuples::const_iterator iter = tuples.begin();

  // print out the arcs, combine if desired
  if (argMulti || !preserve4Tuple)
    while ((iter != tuples.end()) && ((*iter).rsf[0] != RsfTuple::locString)) {
      int numArcs = 0;
      const RsfTuple& rsfTuple = *iter;

      while ((++iter != tuples.end()) &&
	     (argMulti || (rsfTuple.rsf[0] == (*iter).rsf[0])) &&
	     (rsfTuple.rsf[1] == (*iter).rsf[1]) &&
	     (rsfTuple.rsf[2] == (*iter).rsf[2]))
	numArcs++;
      if (argMulti && numArcs) {
	if (!RsfTuple(RsfTuple::multiarcString, rsfTuple.rsf[1],
		      rsfTuple.rsf[2], rsfTuple.rsf[3],
		      rsfTuple.line, rsfTuple.column).print(output, 
							    preserve4Tuple))
	  return 0;
      } else
	if (!rsfTuple.print(output, false))
	  return 0;
    }
  else
    while ((iter != tuples.end()) && ((*iter).rsf[0] != RsfTuple::locString)) {
      const RsfTuple& rsfTuple = *iter;
      if (!rsfTuple.print(output, true))
	return 0;
      ++iter;
    }

  // combine loc attributes
  while (iter != tuples.end()) {
    const RsfTuple& rsfTuple = *iter;
    string loc = rsfTuple.rsf[2].string();
    while ((++iter != tuples.end()) && argLoc &&
	   (rsfTuple.rsf[1] == (*iter).rsf[1])) {
      loc += ";";
      loc += (*iter).rsf[2].string();
    }
    if (!RsfTuple(RsfTuple::locString, rsfTuple.rsf[1],
		  loc, UniqueString(), 0, 0).print(output, false))
      return 0;
  }

  return 1;
}

template <class Tuples> int writeOutput(Tuples& tuples, bool preserve4Tuple,
					bool argLoc, bool argMulti) {
  writeOutput(stdout, tuples, preserve4Tuple, argLoc, argMulti);
}

#endif
