/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Johannes Martin (jmartin@csr.csc.uvic.ca)                   */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include <string.h>

#include "rsftuple.h"

const UniqueString RsfTuple::typeString     = UniqueString("type");
const UniqueString RsfTuple::locString	    = UniqueString("loc");
const UniqueString RsfTuple::multiarcString = UniqueString("multiarc");

RsfTuple::RsfTuple(const UniqueString& elem0, const UniqueString& elem1,
		   const UniqueString& elem2, const UniqueString& elem3,
		   int line, int column) {
  rsf[0] = elem0;
  rsf[1] = elem1;
  rsf[2] = elem2;
  rsf[3] = elem3;
  this->line = line;
  this->column = column;
}

RsfTuple::RsfTuple(char *p, unsigned long lineNo, 
		   bool preserve4Tuple, bool forgiving) {
  int i;

  /* skip leading whitespace */
  p += strspn(p, " \t\n\r");

  /* Display and extract comment lines from the RSF stream */
  if (*p == '#') {
    if (!forgiving)
      fprintf(stderr, "%s", p);
    rsf[0] = "";
    return;
  }

  /* parse elements 1..3 */
  for (i = 0; (*p != '\0') && (i < 3); i++) {
    char *start;

    if (strchr("\"'`", *p)) {
      char quote = *p;
      /* skip over opening quote */
      start = ++p;
      /* Search for a closing quote immediately followed by a white space. */
      do {
	p = strchr(p, quote);
      } while (p != NULL && *(++p) && strchr(" \t\n\r", *p) == NULL);
      /* remove closing quote */
      if (p != NULL)
	*(p - 1) = '\0';
    } else {
      /* Search for a white space. */
      start = p;
      p = strpbrk(p, " \t\n\r");
    }

    if (p == NULL)
      p = "";
    else {
      *p++ = 0;
      p += strspn(p, " \t\n\r");
    }

    rsf[i] = start;
  }

  /* if there are 0 < elements < 3, then there is a problem with the rsf */
  if (i < 3) {
    if (!forgiving && (i > 0)) {
      fprintf(stderr,
	      "RSF tuple has fewer than 3 components at line %lu\n", lineNo);
      rsf[0] = "";
    }
    return;
  }

  if (!preserve4Tuple || (*p == '\0')) {
    rsf[3] = "";
    line   = 0;
    column = 0;
    if ((p = strpbrk(p, " \t\n\r")) == NULL)
      p = "";
  } else {
    char *start = p;
    if (((p = strchr(start, ',')) == NULL) &&
	((p = strpbrk(start, " \t\n\r")) == NULL))
      p = "";
    else
      *p = '\0';
    rsf[3] = start;
    line   = strtol(p + 1, &p, 0);
    column = strtol(p + 1, &p, 0);
  }
  if (!forgiving && (*(p + strspn(p, " \t\n\r")) != '\0'))
    fprintf(stderr, "RSF tuple has more than 4 components at line %lu\n",
	    lineNo);
}

bool RsfTuple::operator< (const RsfTuple& other) const {
  if (rsf[0] == typeString)
    if (other.rsf[0] == typeString)
      for (int i = 1; i < 3; i++) {
	int result = rsf[i].string().compare(other.rsf[i].string());
	if (result < 0)
	  return true;
	if (result > 0)
	  return false;
      }
    else
      return true;
  else
    if (other.rsf[0] == typeString)
      return false;
    else
      if (rsf[0] == locString)
	if (other.rsf[0] == locString)
	  for (int i = 1; i < 3; i++) {
	    int result = rsf[i].string().compare(other.rsf[i].string());
	    if (result < 0)
	      return true;
	    if (result > 0)
	      return false;
	  }
	else
	  return false;
      else
	if (other.rsf[0] == locString)
	  return true;
	else
	  for (int i = 1; i < 4; i++) {
	    int result = rsf[i%3].string().compare(other.rsf[i%3].string());
	    if (result < 0)
	      return true;
	    if (result > 0)
	      return false;
	  }

  if (rsf[3].empty())
    if (other.rsf[3].empty())
      return false;
    else
      return true;
  else
    if (other.rsf[3].empty())
      return false;
    else {
      int i;
      if ((i = rsf[3].string().compare(other.rsf[3].string())) == 0)
	if (line == other.line)
	  return column < other.column;
	else
	  return line < other.line;
      else
	return i < 0;
    }
}

void RsfTuple::printElement(FILE* output, const char *element) {
  if (strpbrk(element, " \t")) {
    char *quote = "\"'`";
    while (strchr(element, *quote) && *(quote + 1))
      quote++;
    fprintf(output, "%c%s%c", *quote, element, *quote);
  } else
    fprintf(output, "%s", element);
}

bool RsfTuple::print(FILE* output, bool preserve4Tuple) const {
  printElement(output, rsf[0]);
  fprintf(output, "\t");
  printElement(output, rsf[1]);
  fprintf(output, "\t");
  printElement(output, rsf[2]);
  if (preserve4Tuple && (rsf[3].string() != "")) {
    fprintf(output, "\t%s", rsf[3].chars());
    if (line != 0) {
      fprintf(output, ",%d", line);
      if (column != 0)
	fprintf(output, ",%d", column);
    }
  }
  return fprintf(output, "\n") > 0;
}

